Attribute VB_Name = "StackErrors"
' Manage an error stack.
Option Explicit

' Error code to represent a stack dump.
Public Const errStackDump = 41001

' The collection of routine information.
Private ErrorStack As Collection

' The error information that started it all.
Private ErrorNumber As Long
Private ErrorSource As String
Private ErrorDescription As String
Private ErrorHelpFile As String
Private ErrorHelpContext As Long

' Create a RoutineInfo object, save the values
' in it, and add it to the top of the ErrorStack
' collection.
Public Sub AddToErrorStack(ByVal reraise As Boolean, ByVal routine_name As String, ParamArray variable_values() As Variant)
Dim routine_info As New RoutineInfo
Dim variable_num As Integer

    ' Make sure we have an even number of
    ' variable_values items because variable
    ' names and values come in pairs.
    If (UBound(variable_values) + 1) Mod 2 <> 0 Then Stop

    ' If this is the first error in the stack,
    ' allocate the stack and save error information.
    If ErrorStack Is Nothing Then ClearErrorStack

    ' Save the RoutineInfo values.
    routine_info.SaveName routine_name

    For variable_num = 0 To UBound(variable_values) Step 2
        routine_info.SaveValue _
            variable_values(variable_num), _
            variable_values(variable_num + 1)
    Next variable_num

    ' Add the new RoutineInfo item to the top of
    ' the ErrorStack stack.
    If ErrorStack.Count = 0 Then
        ErrorStack.Add routine_info
    Else
        ErrorStack.Add routine_info, , 1
    End If

    ' If we should reraise the error, do so.
    If reraise Then _
        Err.Raise ErrorNumber, ErrorSource, _
            ErrorDescription, ErrorHelpFile, _
            ErrorHelpContext
End Sub
' Return a textual representation of the stack.
Public Function ErrorStackText() As String
Dim routine_info As RoutineInfo
Dim result As String
Dim num_spaces As Integer

    ' Make sure there is stack info.
    If ErrorStack Is Nothing Then Stop

    ' Start with the error information.
    result = "Error " & Format$(ErrorNumber) & _
        "." & vbCrLf & Err.Description & vbCrLf & _
        vbCrLf & "Stack dump:" & vbCrLf

    ' Create the entries for each routine.
    For Each routine_info In ErrorStack
        ' Add this routine's information.
        result = result & routine_info.TextValue(num_spaces)
        num_spaces = num_spaces + 4
    Next routine_info

    ErrorStackText = result

    ' Empty the stack.
    Set ErrorStack = Nothing
End Function

' Allocate the stack and save the current error
' information.
Public Sub ClearErrorStack()
    ' Start with a new stack collection.
    Set ErrorStack = New Collection

    ' Save the error values.
    ErrorNumber = Err.Number
    ErrorSource = Err.Source
    ErrorDescription = Err.Description
    ErrorHelpFile = Err.HelpFile
    ErrorHelpContext = Err.HelpContext
End Sub

